// (C) 2019-2024 GoodData Corporation
import { InMemoryPaging, DummySemanticSearchQueryBuilder, DummyGenAIChatThread, } from "@gooddata/sdk-backend-base";
import { NotSupported, } from "@gooddata/sdk-backend-spi";
import { idRef, } from "@gooddata/sdk-model";
import RecordedAttributeHierarchiesService from "./attributeHierarchies.js";
import { RecordedAttributes } from "./attributes.js";
import { RecordedCatalogFactory } from "./catalog.js";
import { RecordedDashboards } from "./dashboards.js";
import { RecordedExecutionFactory } from "./execution.js";
import { RecordedFacts } from "./facts.js";
import { RecordedInsights } from "./insights.js";
import { RecordedMeasures } from "./measures.js";
import { RecordedWorkspaceUsersQuery, recordedAccessControlFactory, recordedUserGroupsQuery, } from "./userManagement.js";
const defaultConfig = {
    hostname: "test",
};
const USER_ID = "recordedUser";
const locale = "en-US";
const separators = {
    thousand: ",",
    decimal: ".",
};
/**
 * @internal
 */
export const defaultRecordedBackendCapabilities = {
    canCalculateGrandTotals: true,
    canCalculateSubTotals: true,
    canCalculateTotals: true,
    canCalculateNativeTotals: true,
    supportsCsvUploader: true,
    supportsKpiWidget: false,
    supportsWidgetEntity: true,
    supportsOwners: true,
    allowsInconsistentRelations: false,
    supportsHierarchicalWorkspaces: false,
    supportsCustomColorPalettes: true,
    supportsElementsQueryParentFiltering: true,
    supportsElementUris: true,
    supportsEveryoneUserGroupForAccessControl: true,
};
/**
 * Creates new backend that will be providing recorded results to the caller. The recorded results are provided
 * to the backend in the form of RecordingIndex. This contains categorized recordings for the different service
 * calls.
 *
 * Note that:
 * - the 'tools/mock-handling' program can be used to create recordings AND the recording index.
 * - typically you want to use this recordedBackend with the recordings from the reference workspace; there
 *   is already tooling and infrastructure around populating that project
 *
 * @param index - recording index
 * @param config - backend config, for now just for compatibility sakes with the analytical backend config
 * @param capabilities - backend capabilities to use
 * @internal
 */
export function recordedBackend(index, config = defaultConfig, capabilities = defaultRecordedBackendCapabilities) {
    const backend = {
        capabilities,
        config,
        onHostname(hostname) {
            return recordedBackend(index, { ...config, hostname });
        },
        withTelemetry(_component, _props) {
            return backend;
        },
        withAuthentication(_) {
            return this;
        },
        organization(organizationId) {
            return recordedOrganization(organizationId, config);
        },
        organizations() {
            return recordedOrganizations(config);
        },
        currentUser() {
            return recordedUserService(config);
        },
        workspace(id) {
            return recordedWorkspace(id, index, config);
        },
        workspaces() {
            throw new NotSupported("not supported");
        },
        authenticate() {
            return Promise.resolve({ userId: USER_ID });
        },
        deauthenticate() {
            return Promise.resolve();
        },
        isAuthenticated() {
            return Promise.resolve({ userId: USER_ID });
        },
        entitlements() {
            return recordedEntitlements();
        },
        dataSources() {
            throw new NotSupported("not supported");
        },
    };
    return backend;
}
function recordedWorkspace(workspace, recordings = {}, implConfig) {
    const insightsService = new RecordedInsights(recordings, implConfig.useRefType ?? "uri");
    return {
        workspace,
        async getDescriptor() {
            return recordedDescriptor(this.workspace, implConfig);
        },
        async updateDescriptor() {
            throw new NotSupported("not supported");
        },
        getParentWorkspace() {
            throw new NotSupported("not supported");
        },
        execution() {
            return new RecordedExecutionFactory(recordings, workspace, implConfig.useRefType ?? "uri");
        },
        attributes() {
            return new RecordedAttributes(recordings, implConfig);
        },
        measures() {
            return new RecordedMeasures();
        },
        facts() {
            return new RecordedFacts();
        },
        insights() {
            return insightsService;
        },
        dashboards() {
            return new RecordedDashboards(this.workspace, insightsService, recordings);
        },
        settings() {
            return {
                async getSettings() {
                    return {
                        workspace,
                        ...(implConfig.globalSettings ?? {}),
                    };
                },
                async getSettingsForCurrentUser() {
                    return {
                        userId: USER_ID,
                        workspace,
                        locale,
                        separators,
                        ...(implConfig.globalSettings ?? {}),
                    };
                },
                async setLocale() {
                    return Promise.resolve();
                },
                async setTimezone() {
                    return Promise.resolve();
                },
                async setDateFormat() {
                    return Promise.resolve();
                },
                async setWeekStart() {
                    return Promise.resolve();
                },
                async setColorPalette() {
                    return Promise.resolve();
                },
                async setTheme() {
                    return Promise.resolve();
                },
                async deleteColorPalette() {
                    return Promise.resolve();
                },
                async deleteTheme() {
                    return Promise.resolve();
                },
            };
        },
        styling() {
            return {
                async getColorPalette() {
                    return implConfig.globalPalette ?? [];
                },
                async getTheme() {
                    return implConfig.theme ?? {};
                },
                async getActiveTheme() {
                    return Promise.resolve(undefined);
                },
                async setActiveTheme() {
                    return Promise.resolve(undefined);
                },
                async getActiveColorPalette() {
                    return Promise.resolve(undefined);
                },
                async setActiveColorPalette() {
                    return Promise.resolve(undefined);
                },
                clearActiveColorPalette() {
                    return Promise.resolve(undefined);
                },
                clearActiveTheme() {
                    return Promise.resolve(undefined);
                },
            };
        },
        dateFilterConfigs() {
            return recordedDateFilterConfig(implConfig);
        },
        catalog() {
            return new RecordedCatalogFactory(workspace, recordings, implConfig);
        },
        datasets() {
            throw new NotSupported("not supported");
        },
        permissions() {
            return recordedPermissionsFactory();
        },
        users() {
            return new RecordedWorkspaceUsersQuery(implConfig);
        },
        userGroups() {
            return recordedUserGroupsQuery(implConfig);
        },
        accessControl() {
            return recordedAccessControlFactory(implConfig);
        },
        attributeHierarchies() {
            return new RecordedAttributeHierarchiesService(implConfig);
        },
        exportDefinitions() {
            throw new NotSupported("not supported");
        },
        dataFilters() {
            throw new NotSupported("not supported");
        },
        logicalModel() {
            throw new NotSupported("not supported");
        },
        automations() {
            throw new NotSupported("not supported");
        },
        genAI() {
            return {
                getChatThread() {
                    return new DummyGenAIChatThread();
                },
                getSemanticSearchQuery: () => {
                    return new DummySemanticSearchQueryBuilder(workspace);
                },
                semanticSearchIndex: () => {
                    throw new NotSupported("not supported");
                },
            };
        },
    };
}
function recordedEntitlements() {
    return {
        resolveEntitlements: () => {
            return Promise.resolve([]);
        },
    };
}
function recordedOrganization(organizationId, implConfig) {
    const scopeFactory = implConfig.securitySettingsOrganizationScope === undefined
        ? (organizationId) => `/gdc/domains/${organizationId}`
        : implConfig.securitySettingsOrganizationScope;
    return {
        organizationId,
        getDescriptor() {
            return Promise.resolve({
                id: organizationId,
                title: "mock organization",
            });
        },
        updateDescriptor() {
            return Promise.resolve({
                id: organizationId,
                title: "mock organization",
            });
        },
        securitySettings() {
            return {
                scope: scopeFactory(organizationId),
                isUrlValid(url, context) {
                    if (implConfig.securitySettingsUrlValidator !== undefined) {
                        return Promise.resolve(implConfig.securitySettingsUrlValidator(url, context));
                    }
                    return Promise.resolve(true);
                },
                isDashboardPluginUrlValid(url, workspace) {
                    if (implConfig.securitySettingsPluginUrlValidator !== undefined) {
                        return Promise.resolve(implConfig.securitySettingsPluginUrlValidator(url, workspace));
                    }
                    return Promise.resolve(true);
                },
            };
        },
        styling() {
            const resolveTheme = (theme) => {
                return Promise.resolve({
                    type: "theme",
                    id: "theme_id",
                    title: "Theme 1",
                    description: "",
                    production: true,
                    deprecated: false,
                    unlisted: false,
                    ref: idRef("theme_id"),
                    uri: "theme_uri",
                    theme: theme.theme,
                });
            };
            const resolveColorPalette = (colorPalette) => Promise.resolve({
                type: "colorPalette",
                id: "color_palette_id",
                title: "Color Palette 1",
                description: "",
                production: true,
                deprecated: false,
                unlisted: false,
                ref: idRef("color_palette_id"),
                uri: "color_palette_uri",
                colorPalette: colorPalette.colorPalette,
            });
            return {
                getThemes: () => Promise.resolve([]),
                getActiveTheme: () => Promise.resolve(undefined),
                setActiveTheme: () => Promise.resolve(),
                clearActiveTheme: () => Promise.resolve(),
                createTheme: resolveTheme,
                updateTheme: resolveTheme,
                deleteTheme: () => Promise.resolve(),
                getColorPalettes: () => Promise.resolve([]),
                getActiveColorPalette: () => Promise.resolve(undefined),
                setActiveColorPalette: () => Promise.resolve(),
                clearActiveColorPalette: () => Promise.resolve(),
                createColorPalette: resolveColorPalette,
                updateColorPalette: resolveColorPalette,
                deleteColorPalette: () => Promise.resolve(),
            };
        },
        settings() {
            return {
                setWhiteLabeling: () => Promise.resolve(),
                setLocale: () => Promise.resolve(),
                setTimezone: () => Promise.resolve(),
                setDateFormat: () => Promise.resolve(),
                setWeekStart: () => Promise.resolve(),
                getSettings: () => Promise.resolve({}),
                setTheme: () => Promise.resolve(),
                setColorPalette: () => Promise.resolve(),
                setOpenAiConfig: () => Promise.resolve(),
                setAlertDefault: () => Promise.resolve(),
                deleteTheme: () => Promise.resolve(),
                deleteColorPalette: () => Promise.resolve(),
            };
        },
        notificationChannels() {
            return {
                getCount: () => Promise.resolve(0),
                getAll: () => Promise.resolve([]),
                deleteChannel: () => Promise.resolve(),
                //emails
                createEmail: (webhook) => Promise.resolve({
                    ...webhook,
                    id: "dummySmtp",
                }),
                deleteEmail: () => Promise.resolve(),
                getEmail: () => Promise.resolve({
                    id: "dummySmtp",
                    type: "smtp",
                    triggers: [],
                    destination: {
                        type: "custom",
                        name: "",
                        address: "",
                        person: "",
                        login: "",
                        password: "",
                        from: "",
                        hasPassword: true,
                        port: 25,
                    },
                }),
                getEmails: () => Promise.resolve([]),
                updateEmail: (smtp) => Promise.resolve(smtp),
                //webhooks
                createWebhook: (webhook) => Promise.resolve({
                    ...webhook,
                    id: "dummyWebhook",
                }),
                deleteWebhook: () => Promise.resolve(),
                getWebhook: () => Promise.resolve({
                    id: "dummyWebhook",
                    type: "webhook",
                    destination: {
                        name: "Dummy webhook",
                        endpoint: "https://dummy.webhook",
                        token: "dummyToken",
                    },
                    triggers: [],
                }),
                getWebhooks: () => Promise.resolve([]),
                updateWebhook: (webhook) => Promise.resolve(webhook),
            };
        },
        permissions() {
            return {
                getOrganizationPermissionForUser: () => Promise.resolve([]),
                getOrganizationPermissionForUserGroup: () => Promise.resolve([]),
                updateOrganizationPermissions: () => Promise.resolve(),
                getPermissionsForUser: () => Promise.resolve({ workspacePermissions: [], dataSourcePermissions: [] }),
                getPermissionsForUserGroup: () => Promise.resolve({ workspacePermissions: [], dataSourcePermissions: [] }),
                assignPermissions: () => Promise.resolve(),
                revokePermissions: () => Promise.resolve(),
            };
        },
        users() {
            return {
                createUser: () => {
                    throw new NotSupported("not supported");
                },
                getUsersQuery: () => {
                    throw new NotSupported("not supported");
                },
                getUserGroupsQuery: () => {
                    throw new NotSupported("not supported");
                },
                addUsersToUserGroups: () => Promise.resolve(),
                createUserGroup: () => Promise.resolve(),
                deleteUsers: () => Promise.resolve(),
                deleteUserGroups: () => Promise.resolve(),
                getUser: () => Promise.resolve(undefined),
                getUserGroup: () => Promise.resolve(undefined),
                getUserGroups: () => Promise.resolve([]),
                getUserGroupsOfUser: () => Promise.resolve([]),
                getUsersOfUserGroup: () => Promise.resolve([]),
                getUsers: () => Promise.resolve([]),
                removeUsersFromUserGroups: () => Promise.resolve(),
                updateUser: () => Promise.resolve(),
                updateUserGroup: () => Promise.resolve(),
            };
        },
        llmEndpoints() {
            const dummyEndpoint = {
                id: "dummyLlmEndpoint",
                title: "Dummy Llm Endpoint",
                provider: "OPENAI",
                model: "gpt-4o-mini",
            };
            return {
                getCount: () => Promise.resolve(0),
                getAll: () => Promise.resolve([]),
                deleteLlmEndpoint: () => Promise.resolve(),
                getLlmEndpoint: () => Promise.resolve({
                    ...dummyEndpoint,
                }),
                createLlmEndpoint: () => Promise.resolve({
                    ...dummyEndpoint,
                }),
                updateLlmEndpoint: () => Promise.resolve({
                    ...dummyEndpoint,
                }),
                patchLlmEndpoint: () => Promise.resolve({
                    ...dummyEndpoint,
                }),
            };
        },
    };
}
function recordedOrganizations(implConfig) {
    return {
        getCurrentOrganization() {
            return Promise.resolve(recordedOrganization("mock-organization", implConfig));
        },
    };
}
// returns the same settings as the global ones
function recordedUserService(implConfig) {
    return {
        async getUser() {
            return (implConfig.userManagement?.user ?? {
                login: USER_ID,
                ref: idRef(USER_ID),
                email: "",
                fullName: "",
                firstName: "",
                lastName: "",
            });
        },
        settings() {
            return {
                getSettings: async () => ({
                    userId: USER_ID,
                    locale,
                    separators,
                    ...(implConfig.globalSettings ?? {}),
                }),
                setLocale: () => Promise.resolve(),
            };
        },
    };
}
// return true for all
function recordedPermissionsFactory() {
    return {
        getPermissionsForCurrentUser: async () => ({
            canAccessWorkbench: true,
            canCreateAnalyticalDashboard: true,
            canCreateReport: true,
            canCreateVisualization: true,
            canExecuteRaw: true,
            canExportReport: true,
            canExportTabular: true,
            canExportPdf: true,
            canInitData: true,
            canManageAnalyticalDashboard: true,
            canManageMetric: true,
            canManageProject: true,
            canManageReport: true,
            canUploadNonProductionCSV: true,
            canCreateScheduledMail: true,
            canListUsersInProject: true,
            canManageDomain: true,
            canInviteUserToProject: true,
            canRefreshData: true,
            canManageACL: true,
            canManageScheduledMail: true,
            canCreateFilterView: true,
            canCreateAutomation: true,
        }),
    };
}
function recordedDescriptor(workspaceId, implConfig) {
    const { title, description, isDemo } = implConfig.workspaceDescriptor || {};
    return {
        id: workspaceId,
        title: title ?? "",
        description: description ?? "",
        isDemo: isDemo ?? false,
    };
}
function recordedDateFilterConfig(implConfig) {
    return {
        withLimit(_limit) {
            return this;
        },
        withOffset(_offset) {
            return this;
        },
        query() {
            const { dateFilterConfig } = implConfig;
            return Promise.resolve(new InMemoryPaging(dateFilterConfig ? [dateFilterConfig] : []));
        },
    };
}
//# sourceMappingURL=index.js.map